@@ -525,7 +525,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
             template <class A1, class A2>
             constexpr auto operator()(const A1& t1, const A2& t2) const noexcept
             {
-                return (t1 < t2) ? t1 : t2;
+                return xtl::select(t1 < t2, t1, t2);
             }
 
             template <class A1, class A2>
@@ -541,7 +541,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
             template <class A1, class A2>
             constexpr auto operator()(const A1& t1, const A2& t2) const noexcept
             {
-                return (t1 > t2) ? t1 : t2;
+                return xtl::select(t1 > t2, t1, t2);
             }
 
             template <class A1, class A2>
@@ -556,7 +556,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
             template <class A1, class A2, class A3>
             constexpr auto operator()(const A1& v, const A2& lo, const A3& hi) const
             {
-                return v < lo ? lo : hi < v ? hi : v;
+                return xtl::select(v < lo, lo, xtl::select(hi < v, hi, v));
             }
 
             template <class A1, class A2, class A3>
@@ -921,8 +921,7 @@ XTENSOR_INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);
     template <class F, class... E>
     inline auto make_lambda_xfunction(F&& lambda, E&&... args)
     {
-        using xfunction_type = xfunction<detail::lambda_adapt<F>,
-                                         const_xclosure_t<E>...>;
+        using xfunction_type = typename detail::xfunction_type<detail::lambda_adapt<F>, E...>::type;
         return xfunction_type(detail::lambda_adapt<F>(std::forward<F>(lambda)), std::forward<E>(args)...);
     }
 
